//输换
var html2json = require('html2json').html2json;
var json2html = require('html2json').json2html;

var fs = require('fs'),
    path = require('path');
var debug = require('debug')('wevue');

var tagNames = {
    'view': 'div',
    'form': 'form',
    'input': 'input',
    'button': 'button',
    'picker': 'picker',
    'image': 'img',
    'text': 'span'
};

//必转
var attrNames = {
    'wx:if': 'v-if',
    'wx:else': 'v-else',
    'wx:elif': 'v-else-if',
    'wx:for': 'v-for',
    'wx:for-item': 'v-for-item',
    'wx:for-index': 'v-for-index',
    'wx:key': 'v-bind:key'
}


function tagNameProcessors(name) {
    debug('tag', name);
    return tagNames[name] || name;
}

function valueProcessors(value) {
    return value;
}

function tagAttrProcessors(attr) {
    var newAttrs = {};
    for (var name in attr) {
        if (attr.hasOwnProperty(name)) {
            var newName = attrNameProcessors(attr[name], name);
            var newValue = attrValueProcessors(attr[name], name, attr);

            debug('newName', newName);
            debug('newValue', newValue);
            newAttrs[newName] = newValue;
        }
    }
    return newAttrs;
}

var util = require('util');
//处理属性名称
function attrNameProcessors(value, name, node) {
    var name = name.toLowerCase();
    debug('value=', value, 'name=', name);
    try {
        if (attrNames[name]) {
            return attrNames[name];
        } else if (util.isArray(value) && /{{.*}}/.test(value.join(''))) {
            return 'v-bind:' + name;
        } else if (/{{.*}}/.test(value)) {
            return 'v-bind:' + name;
        } else {
            return name;
        }

    } catch (e) {
        console.log(e.stack);
    }
}

//处理属性值
//src="abc{{value}}"   =>   v-bind:src="  'abc' + value"
//src="{{value}}abc"   =>   v-bind:src="   value+'abc'"
//src="abc{{value}}efg"   =>   v-bind:src=" 'abc' + value+'efg'"
function attrValueProcessors(value, name, attributes) {
    var newValue = [];
    try {
        debug('attrValue', name, value, typeof(value), value.length);
        if (!value) {
            return value;
        }


        var ret = "";
        switch (name) {
            case 'wx:for':
                var val = util.isArray(value) ? value.join('') : value;
                debug(val);
                val = attrItemValueProcessors(val);
                var item = attributes['wx:for-item'];
                var index = attributes['wx:for-index'] || 'index';
                if (item && index) {
                    ret = `(${index},${item}) in ${val}`;
                } else if (item) {
                    ret = `${item} in ${val}`;
                } else if (index) {
                    ret = `(${index},item) in ${val}`;
                } else {
                    ret = `for ${item} in ${val}`;
                }
                newValue[0] = ret;
                break;
            case 'wx:for-index':
            case 'wx:for-item':
                newValue = value;
                break;
            case 'wx:key':
                newValue = value;
                if (value == "*this") {
                    newValue = "index"
                }
                break;
            case 'wx:if':
            case 'wx:elif':
            case 'wx:else':
                var val = util.isArray(value) ? value.join('') : value;
                newValue = attrItemValueProcessors(val);
                break;
            default:
                if (util.isArray(value)) {
                    for (var i = 0; i < value.length; i++) {
                        var val = value[i];
                        debug('val', val);
                        newValue[i] = attrItemValueProcessors(val);
                    }
                } else {
                    newValue = attrItemValueProcessors(value);
                }
        }
    } catch (e) {
        console.log(name, '=', value);
        console.log(e.stack);
    }
    return newValue;
}


function attrItemValueProcessors(val) {
    if (/{{.*}}/.test(val)) {
        //处理带绑定属性值
        var matches = val.match(/(.*){{(.*)}}(.*)/);
        debug(matches, matches.length);
        if (matches && matches.length === 4) {
            var left = matches[1];
            var exp = matches[2];
            var right = matches[3];
            if (left && exp && right) {
                val = `'${left}'+ ${exp} + '${right}'`;
            } else if (left && exp) {
                val = `'${left}' + ${exp}`;
            } else if (right) {
                val = `${exp} + '${right}'`;
            } else {
                val = exp;
            }
        }
    }
    return val;
}

var vue;

function parser(wxml, off,cb) {
    if (off){
      //关闭转换
      return cb(null, wxml);
    }

    vue = {
        node: 'root',
        child: []
    };
    debug(wxml);

    wxml = wxml.replace(/<!--[\s\S]*?-->/g, "");
    try {
        var json = html2json(wxml);
        wxml2vue(json);
        var vueHtml = json2html(vue);
        return cb(null, vueHtml);
    } catch (e) {
        //console.log(wxml);
        console.log(e.stack);
        debug("can not parser", wxml);
        return cb(null, wxml);
    } finally {

    }




}

function wxml2vue(json) {
    if (json.node === 'root') {
        debug(json.child.length);
        for (var i = 0; i < json.child.length; i++) {
            var ele = json.child[i];
            var newElement = null;
            if (ele && ele.child) {
                debug('==>', i, ele.tag);
                newElement = {
                    "node": ele.node,
                    "tag": tagNameProcessors(ele.tag),
                    "attr": tagAttrProcessors(ele.attr),
                    "text": ele.text,
                    "child": []
                };
                eachElement(ele.child, newElement)
            } else {
                debug('==>', i, ele);
                newElement = {
                    "node": ele.node,
                    "tag": tagNameProcessors(ele.tag),
                    "attr": tagAttrProcessors(ele.attr),
                    "text": ele.text,
                };
            }

            vue.child[i] = newElement;
        }
    }
}

function eachElement(elements, newElement) {
    for (var i = 0; i < elements.length; i++) {
        var ele = elements[i];
        var newEle;
        if (ele && ele.child) {
            newEle = {
                "node": ele.node,
                "tag": tagNameProcessors(ele.tag),
                "attr": tagAttrProcessors(ele.attr),
                "text": ele.text,
                "child": []
            };
            debug('=====>>', i, ele.tag);
            eachElement(ele.child, newEle);
        } else {
            debug('====>', i, ele);
            newEle = {
                "node": ele.node,
                "tag": tagNameProcessors(ele.tag),
                "attr": tagAttrProcessors(ele.attr),
                "text": ele.text
            };
        }
        newElement.child[i] = newEle;
    }
}
module.exports = parser;
